#include "OperatorTerm.h"
#include "ElInfo.h"
#include "DOFVector.h"

namespace AMDiS {

  const Flag OperatorTerm::PW_CONST = 1;
  const Flag OperatorTerm::SYMMETRIC = 2;


  void OperatorTerm::setSymmetric(bool symm)
  {
    if (symm)
      properties.setFlag(SYMMETRIC);
    else
      properties.unsetFlag(SYMMETRIC);       
  }


  bool OperatorTerm::isSymmetric()
  {
    return properties.isSet(SYMMETRIC);
  }


  double *OperatorTerm::getVectorAtQPs(DOFVectorBase<double>* vec,
				       const ElInfo* elInfo, 
				       SubAssembler* subAssembler,
				       Quadrature *quad)
  {
    FUNCNAME("OperatorTerm::getVectorAtQPs()");

    TEST_EXIT_DBG(elInfo->getMesh() == vec->getFESpace()->getMesh())
      ("There is something wrong!\n");
      
    return subAssembler->getVectorAtQPs(vec, elInfo, quad);
  }


  double *OperatorTerm::getVectorAtQPs(DOFVectorBase<double>* vec,
				       const ElInfo* smallElInfo, 
				       const ElInfo* largeElInfo, 
				       SubAssembler* subAssembler,
				       Quadrature *quad)
  {
    FUNCNAME("OperatorTerm::getVectorAtQPs()");

    TEST_EXIT(smallElInfo->getMesh() == vec->getFESpace()->getMesh() ||
	      largeElInfo->getMesh() == vec->getFESpace()->getMesh())
      ("There is something wrong!\n");

    if (smallElInfo->getLevel() == largeElInfo->getLevel()) {

      // Both elements have the same size, so we can use the simple procedure
      // to determine the vecAtQPs.
      
      if (vec->getFESpace()->getMesh() == smallElInfo->getMesh())
	return subAssembler->getVectorAtQPs(vec, smallElInfo, quad);
      else
	return subAssembler->getVectorAtQPs(vec, largeElInfo, quad);      

    } else {

      // The two elements are different. If the vector is defined on the mesh of the
      // small element, we can still use the simple procedure to determine the vecAtQPs.

      if (vec->getFESpace()->getMesh() == largeElInfo->getMesh())
	return subAssembler->getVectorAtQPs(vec, smallElInfo, largeElInfo, quad);
      else
	return subAssembler->getVectorAtQPs(vec, smallElInfo, quad);
    }
  }


  WorldVector<double>* OperatorTerm::getGradientsAtQPs(DOFVectorBase<double>* vec,
						       const ElInfo* elInfo,
						       SubAssembler* subAssembler,
						       Quadrature *quad)
  {
    FUNCNAME("OperatorTerm::getGradientsAtQPs()");

    TEST_EXIT_DBG(elInfo->getMesh() == vec->getFESpace()->getMesh())
      ("There is something wrong!\n");

    return subAssembler->getGradientsAtQPs(vec, elInfo, quad);
  }


  WorldVector<double>* OperatorTerm::getGradientsAtQPs(DOFVectorBase<double>* vec,
						       const ElInfo* smallElInfo, 
						       const ElInfo* largeElInfo,
						       SubAssembler* subAssembler,
						       Quadrature *quad)
  {
    FUNCNAME("OperatorTerm::getGradientsAtQPs()");

    ERROR_EXIT("Not yet tested!\n");

    TEST_EXIT(smallElInfo->getMesh() == vec->getFESpace()->getMesh() ||
	      largeElInfo->getMesh() == vec->getFESpace()->getMesh())
      ("There is something wrong!\n");

    if (smallElInfo->getLevel() == largeElInfo->getLevel()) {

      // Both elements have the same size, so we can use the simple procedure
      // to determine the gradients.
      
      if (vec->getFESpace()->getMesh() == smallElInfo->getMesh())
	return subAssembler->getGradientsAtQPs(vec, smallElInfo, quad);
      else
	return subAssembler->getGradientsAtQPs(vec, largeElInfo, quad);      

    } else {

      // The two elements are different. If the vector is defined on the mesh of the
      // small element, we can still use the simple procedure to determine the gradients.

      if (vec->getFESpace()->getMesh() == largeElInfo->getMesh())
	return subAssembler->getGradientsAtQPs(vec, smallElInfo, largeElInfo, quad);
      else
	return subAssembler->getGradientsAtQPs(vec, smallElInfo, quad);
    }
  }


}
